@@ -24,6 +24,24 @@ GLOVE_URL_TEMPLATE = 'http://nlp.stanford.edu/data/' + GLOVE_ZIP_FILENAME_TEMPLA
 GLOVE_FILENAME_TEMPLATE = 'glove.{size}B.{dim}d.txt'
 
 
+def glove_normalize(s):
+    """ GloVe deletes whitespace instead of replacing with "_" (Word2Vec style)
+
+    >>> glove_normalize('Los Angeles')
+    'losangeles'
+    """
+    return re.sub(r'\s+', '', s).lower()
+
+
+def word2vec_normalize(s):
+    """ Word2Vec replaces whitespace with "_"
+
+    >>> word2vec_normalize('Los Angeles')
+    'Los_Angeles'
+    """
+    return re.sub(r'\s+', '_', s)
+
+
 def unzip(zip_filepath, dest_filename=None, dim=50):
     """ Extract txt files from ZipFile and place them all in the dest_filepath or DATA_DIR """
     zip_filepath = Path(zip_filepath)
@@ -75,7 +93,7 @@ def download_glove(size=6, url=None, dest_filepath=None):
 def load_glove(dim=50, size=6, filepath=None):
     """ Download and return the specified GloVe word vectors dict from Stanford
 
-    >>> wv = load_glove(dim=100, size=6, filepath='.')
+    >>> wv = load_glove(dim=50, size=6)
     >>> len(wv)
     400000
     """
@@ -108,21 +126,34 @@ def check_load_glove():
     return wv
 
 
+def normalize_vector(x):
+    """ Convert to 1-D np.array and divide vector by it's length (2-norm)
+
+    >>> normalize_vector([0, 1, 0])
+    array([0., 1., 0.])
+    >>> normalize_vector([1, 1, 0])
+    array([0.707..., 0.707..., 0...])
+    """
+    # x = np.array(x).flatten()
+    xnorm = np.linalg.norm(x) or 1
+    xnorm = xnorm if np.isfinite(xnorm) else 1
+    return x / xnorm
+
+
 def cosine_similarity(a, b):
     """ 1 - cos(angle_between(a, b))
 
     >>> cosine_similarity([0, 1, 1], [0, 1, 0])  # 45 deg
-    .707...
+    0.707...
     """
-    a = np.array(a).flatten()
-    b = np.array(b).flatten()
-    a /= np.linalg.norm(a)
-    b /= np.linalg.norm(b)
+    a = normalize_vector(np.array(a).flatten())
+    b = normalize_vector(np.array(b).flatten())
     return (a * b).sum()
 
 
 class IndexedVectors:
-    def __init__(self, vectors=None, index=None):
+    def __init__(self, vectors=None, index=None, normalizer=glove_normalize):
+        self.normalizer = normalizer
         if vectors is None:
             self.load()
         elif isinstance(vectors, dict):
@@ -144,17 +175,12 @@ class IndexedVectors:
             return self.df[key]
         except KeyError:
             print(f"Unable to find '{key}' in {self.df.shape} DataFrame of vectors")
-        lowered_key = key.lower()
-        try:
-            return self.df[lowered_key]
-        except KeyError:
-            print(f"Unable to find '{lowered_key}' in {self.df.shape} DataFrame of vectors")
-        normalized_key = lowered_key.strip().replace('_', ' ')
+        normalized_key = self.normalizer(str(key))
         try:
             return self.df[normalized_key]
         except KeyError:
-            pass
-        raise(KeyError(f"Unable to find any of {set([key, lowered_key, normalized_key])} in self.df.shape DataFrame of vectors"))
+            print(f"Unable to find '{normalized_key}' in {self.df.shape} DataFrame of vectors")
+        raise(KeyError(f"Unable to find any of {set([key, normalized_key])} in self.df.shape DataFrame of vectors"))
 
     def keys(self):
         return self.df.columns.values
